# -*- coding: utf-8 -*-

# Run this app with `python visualization.py` and
# visit http://127.0.0.1:8050/ in your web browser.

import json
import argparse

import dash
import dash_auth

import os,sys,inspect

from data_loader import es_premium_df, role_ids, translate_freq, segments, schools_names, date_column, conn, non_assistent_teachers, file_folder, users_df, vacation_info
import layout_factory

import engagement

import dash_core_components as dcc
import dash_html_components as html
from dash.dependencies import Input, Output

# adding to pythonpath the folder cointaining the camino data science modules 
current_dir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parent_dir = os.path.dirname(current_dir)
sys.path.insert(0, parent_dir)

users_filepath = os.path.join(file_folder,"users.json")
known_users = open(users_filepath, "r")
s = known_users.read()
VALID_USERNAME_PASSWORD_PAIRS = json.loads(s)

app = dash.Dash(__name__, meta_tags=[{"name": "viewport", "content": "width=device-width"}], suppress_callback_exceptions=True)
auth = dash_auth.BasicAuth(
    app,
    VALID_USERNAME_PASSWORD_PAIRS
)

chart_view = engagement.layout.chart_view
input_controls = engagement.layout.inputs
engagement_layout = layout_factory.generate_layout(input_controls, chart_view)
engagement.callbacks.register_callbacks(app)

app.layout = html.Div([
    dcc.Location(id='url', refresh=False),
    html.Div(id='page-content')
])

@app.callback(Output('page-content', 'children'),
              [Input('url', 'pathname')])
def display_page(pathname):
    if pathname == '/engagement':
        return engagement_layout
    elif pathname == '/nps':
        return layout_factory.generate_layout([], [])
    else:
        return '404'

server = app.server

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument("-d","--debug", action="store_true", help="Run dashboard in debug mode")
    args = parser.parse_args()

    debug = args.debug

    app.run_server(debug=debug)