import json
from tqdm import tqdm
from datetime import datetime, timedelta
import pandas as pd
import numpy as np
import sqlalchemy
from sqlalchemy import create_engine
import matplotlib.pyplot as plt
import seaborn as sns
from pathlib import Path

import os,sys,inspect

current_dir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parent_dir = os.path.dirname(current_dir)
sys.path.insert(0, parent_dir)

from Modules.sql_utils import get_all_users_but_dummies_df,postgres_str_from_config
from Modules.dump_es_base import update_dataframe
from Modules.csv_utils import prepare_report
from Modules.schools_helpers import freemium_schools, premium_schools, filter_schools
from Modules.schools_helpers import get_filename, get_folder_path, school_name, role_name
from Modules.time_series_helpers import TimeSeriesSampler
from Modules.cohort_tiers import get_df_for_plotting

import pathlib, os
file_folder = pathlib.Path(__file__).parent.absolute()

# configure SQL connection
sql_config_filepath = os.path.join(file_folder, "..", "sql_config.json")
config_file = open(sql_config_filepath, "r")
s = config_file.read()
config = json.loads(s)

postgres_str = postgres_str_from_config(**config)

conn = create_engine(postgres_str)

# caching common variables
date_column = "date_month_year_day"
ei = list(range(14, 18))
fund1 = [i for i in range(2, 7)]
fund2 = [i for i in range(7, 11)]

users_df = get_all_users_but_dummies_df(conn)
ei_df = users_df[users_df.grade_id.isin(ei)]
fund1_df = users_df[users_df.grade_id.isin(fund1)]
fund2_df = users_df[users_df.grade_id.isin(fund2)]
fund_df = users_df[users_df.grade_id.isin(fund1+fund2)]
groupless_df = users_df[users_df.grade_id.isna()]

# loading pre cached es data base
dtype = {
    'eventProperties.tagged_users_ids': 'string',
    'event_properties.connection_action_type': 'string',
    'user_properties.local_id': 'string', 
    'user_properties.platform': 'string',
    'user_properties.user_agent': 'string'
}
es_premium_csv_path = os.path.join(file_folder, "..", "es_dumps", "es_premium_df.csv")
es_freemium_csv_path = os.path.join(file_folder, "..", "es_dumps", "es_freemium_df.csv")
es_premium_df = pd.read_csv(es_premium_csv_path, dtype=dtype)
es_freemium_df = pd.read_csv(es_freemium_csv_path, dtype=dtype, low_memory=False)
freemium_set = set(freemium_schools)
premium_set = set(premium_schools)

school_as_list = list(map(lambda x: [x], premium_schools))
schools_names = dict(zip(list(map(lambda x: school_name(x, conn), premium_schools)), school_as_list))
schools_names["Agregado: Todas da base"] = premium_schools
role_ids = {
    "professor": 2,
    "aluno": 3
}

translate_freq = {
    "semanal": "weekly",
    "bi-semanal": "biweekly",
    "mensal": "monthly"
}

segments = {
    "Fundamental 1": fund1_df,
    "Fundamental 2": ei_df,
    "Ensino infantil": ei_df,
    "Não vinculados": groupless_df
}

events = es_premium_df.drop_duplicates(subset=["event_type"]).event_type.to_list()

teachers_info_filepath = os.path.join(file_folder,"teachers.csv")
teachers_info = pd.read_csv(teachers_info_filepath)
non_assistent_teachers = teachers_info[teachers_info["Código"] == 1].ID.to_list()

vacation_info_filepath = os.path.join(file_folder,"vacation.csv")
vacation_info = pd.read_csv(vacation_info_filepath)